/*******************************************************************************
**3456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789
**      10        20        30        40        50        60        70        80
**
** gloobus-preview
**
** gloobus-preview-interface-media.cpp - interface for audio/video plugins
**
** Copyright (c) 2010 Gloobus Developers
**
** Authors:
**    Alexandr Grigorcea <cahr.gr@gmail.com>
**
** This program is free software: you can redistribute it and/or modify it
** under the terms of the GNU General Public License version 3, as published
** by the Free Software Foundation.
**
** This program is distributed in the hope that it will be useful, but
** WITHOUT ANY WARRANTY; without even the implied warranties of
** MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
** PURPOSE.  See the GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License along
** with this program.  If not, see <http://www.gnu.org/licenses/>.
**
*******************************************************************************/

#include "gloobus-preview-interface-media.h"

// GNOME MEDIA KEYS HANDLING ================================================ //
#include <dbus/dbus-glib.h>

static void dbus_proxy_destroy_cb (DBusGProxy *proxy, gpointer data);
static void media_key_pressed_cb (DBusGProxy *proxy, const gchar *application, const gchar *key, gpointer data);
static void media_marshal_VOID__STRING_STRING (GClosure     *closure,
                                   GValue       *return_value G_GNUC_UNUSED,
                                   guint         n_param_values,
                                   const GValue *param_values,
                                   gpointer      invocation_hint G_GNUC_UNUSED,
                                   gpointer      marshal_data);
// ========================================================================== //

iMedia::iMedia( void )
	:playing(false)
	,pix_play(0)
	,pix_pause(0)
	,timeout_id(0)
	,f_visible(false)
	,duration(0)
	,dbus_proxy(0)
	,focus_in_id(0)
{
	g_debug("Creating iMedia");
	activate_media_keys(); //TODO: is it the best place to do it?
}

iMedia::~iMedia(void)
{
	g_debug("Destroying iMedia");
	deactivate_media_keys();
	if(pix_play)
		g_object_unref(pix_play);
	if(pix_pause)
		g_object_unref(pix_pause);
}

void iMedia::draw_toolbar( void )
{
	GtkWidget *toolbar = ui->get_toolbar();
	GloobusTheme *theme = ui->get_theme();
	int isize = theme->get_toolbar_icon_size();

	GtkWidget *hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 5);
	GtkWidget *btn_fullscreen = ui->create_button("fullscreen", isize);
	
	duration_label = theme->get_label("--:--");
	position_label = theme->get_label("--:--");

	if(!pix_play)
		pix_play = theme->get_icon("play", isize);
	if(!pix_pause)
		pix_pause = theme->get_icon("pause", isize);

	control_image = gtk_image_new_from_pixbuf(pix_play);
	GtkWidget * btn_control = gtk_event_box_new();
	gtk_event_box_set_visible_window( GTK_EVENT_BOX(btn_control) , false );
	gtk_container_add( GTK_CONTAINER(btn_control) , control_image );

	seekbar = gtk_scale_new_with_range( GTK_ORIENTATION_HORIZONTAL, 0, 1, 1 );
	gtk_scale_set_draw_value(GTK_SCALE(seekbar), false);

	gtk_box_pack_start(GTK_BOX(hbox), btn_control, false, false, 0);
	gtk_box_pack_start(GTK_BOX(hbox), position_label, false, false, 0);
	gtk_box_pack_start(GTK_BOX(hbox), seekbar, true, true, 0);
	gtk_box_pack_start(GTK_BOX(hbox), duration_label, false, false, 0);
	gtk_box_pack_start(GTK_BOX(hbox), btn_fullscreen, false, false, 0);

	g_signal_connect( seekbar, "change-value", G_CALLBACK(seekbar_change_cb), this);
	g_signal_connect( seekbar, "button-press-event", G_CALLBACK(seekbar_press_cb), this);
	g_signal_connect( seekbar, "button-release-event", G_CALLBACK(seekbar_release_cb), this);
	g_signal_connect( btn_control, "button-press-event", G_CALLBACK(button_play_pause), this);
	g_signal_connect( btn_fullscreen, "button-press-event", G_CALLBACK(fullscreen_cb), this);

	gtk_container_add(GTK_CONTAINER(toolbar), hbox);
	gtk_widget_show_all(toolbar);
}

void iMedia::draw_floating_toolbar( void )
{
	GtkWidget *lpad = gtk_alignment_new(0,0,0,0);
	gtk_widget_set_size_request(lpad, 20,-1);
	GtkWidget *rpad = gtk_alignment_new(0,0,0,0);
	gtk_widget_set_size_request(rpad, 20,-1);
	
	GtkWidget *toolbar = ui->get_floating_toolbar();
	GloobusTheme *theme = ui->get_theme();
	int isize = theme->get_toolbar_icon_size();

	GtkWidget *hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 5);
	GtkWidget *btn_fullscreen = ui->create_button("fullscreen", isize);

	f_duration_label = theme->get_label("--:--");
	f_position_label = theme->get_label("--:--");

	if(!pix_play)
		pix_play = theme->get_icon("play", isize);
	if(!pix_pause)
		pix_pause = theme->get_icon("pause", isize);

	if(playing)
		f_control_image = gtk_image_new_from_pixbuf(pix_pause);
	else
		f_control_image = gtk_image_new_from_pixbuf(pix_play);
	GtkWidget * btn_control = gtk_event_box_new();
	gtk_event_box_set_visible_window( GTK_EVENT_BOX(btn_control) , false );
	gtk_container_add( GTK_CONTAINER(btn_control) , f_control_image );

	f_seekbar = gtk_scale_new_with_range( GTK_ORIENTATION_HORIZONTAL, 0, 1, 1 );
	gtk_scale_set_draw_value(GTK_SCALE(f_seekbar), false);

	gtk_box_pack_start(GTK_BOX(hbox), lpad, false, false, 0);
	gtk_box_pack_start(GTK_BOX(hbox), btn_control, false, false, 0);
	gtk_box_pack_start(GTK_BOX(hbox), f_position_label, false, false, 0);
	gtk_box_pack_start(GTK_BOX(hbox), f_seekbar, true, true, 0);
	gtk_box_pack_start(GTK_BOX(hbox), f_duration_label, false, false, 0);
	gtk_box_pack_start(GTK_BOX(hbox), btn_fullscreen, false, false, 0);
	gtk_box_pack_start(GTK_BOX(hbox), rpad, false, false, 0);

	g_signal_connect( f_seekbar, "change-value", G_CALLBACK(seekbar_change_cb), this);
	g_signal_connect( f_seekbar, "button-press-event", G_CALLBACK(seekbar_press_cb), this);
	g_signal_connect( f_seekbar, "button-release-event", G_CALLBACK(seekbar_release_cb), this);
	g_signal_connect( btn_control, "button-press-event", G_CALLBACK(button_play_pause), this);
	g_signal_connect( btn_fullscreen, "button-press-event", G_CALLBACK(fullscreen_cb), this);
	g_signal_connect( toolbar, "unrealize", G_CALLBACK(f_unrealize_cb), this);

	gtk_container_add(GTK_CONTAINER(toolbar), hbox);
	gtk_widget_show_all(hbox);

	f_visible = true;
	update_duration(get_duration());
}

void iMedia::update_position( gint sec )
{
	if( seekbar ) {
		gtk_range_set_value(GTK_RANGE(seekbar), sec );
		if(f_visible)
			gtk_range_set_value(GTK_RANGE(f_seekbar), sec );
	}
		
	if( position_label ) {
		gchar *str = sec_to_str( sec );
		gtk_label_set_text(GTK_LABEL(position_label), str);
		if(f_visible)
			gtk_label_set_text(GTK_LABEL(f_position_label), str);
		g_free(str);
	}
}

void iMedia::update_duration( gint sec )
{
	duration = sec;
	if( duration_label ) {
		gchar *str = sec_to_str( sec );
		gtk_label_set_text(GTK_LABEL(duration_label), str);
		if(f_visible)
			gtk_label_set_text(GTK_LABEL(f_duration_label), str);
		g_free(str);
	}
	if(seekbar && sec > 0) {
		gtk_range_set_range(GTK_RANGE(seekbar), 0, sec);
		if(f_visible)
			gtk_range_set_range(GTK_RANGE(f_seekbar), 0, sec);
	}
}

gchar * iMedia::sec_to_str( gint sec )
{
	gchar *str;
	if( sec <= 0) {
		str = g_strdup("00:00");
	} else if( sec >= 3600) {
		int h = sec / 3600;
		sec = sec % 3600;
		int m = sec / 60;
		sec = sec % 60;
		str = g_strdup_printf("%02d:%02d:%02d", h, m, sec);
	} else {
		int m = sec / 60;
		sec = sec % 60;
		str = g_strdup_printf("%02d:%02d", m, sec);
	}
	return str;
}

void iMedia::set_playing( bool p )
{
	if( p == playing )
		return;

	playing = p;

	if(playing) {
		gtk_image_set_from_pixbuf(GTK_IMAGE(control_image), pix_pause);
        block_slideshow(true);     //Block Slideshow till finished
		if(f_visible)
			gtk_image_set_from_pixbuf(GTK_IMAGE(f_control_image), pix_pause);
	} else {
		gtk_image_set_from_pixbuf(GTK_IMAGE(control_image), pix_play);
        block_slideshow(false);
		if(f_visible)
			gtk_image_set_from_pixbuf(GTK_IMAGE(f_control_image), pix_play);
	}
}

void iMedia::start_timeout( void )
{
	if( !timeout_id )
		timeout_id = g_timeout_add(333, &timeout_cb, this);
}

void iMedia::stop_timeout( void )
{
	if(timeout_id) {
		g_source_remove(timeout_id);
		timeout_id = 0;
	}
}

void iMedia::button_play_pause(GtkWidget *widget,
	GdkEventButton *event,
	gpointer data)
{
	iMedia* media = (iMedia*)data;
	media->set_playing(media->play_pause());
}

gboolean iMedia::seekbar_press_cb(GtkWidget *widget,
	GdkEventButton *event,
	gpointer data)
{
	/* HACK: we want the behaviour you get with the middle button, so we
	 * mangle the event.  clicking with other buttons moves the slider in
	 * step increments, clicking with the middle button moves the slider to
	 * the location of the click.
	 * thanks to totem devs :D
	 */
	event->button = 2;
	return false;
}

gboolean iMedia::seekbar_release_cb(GtkWidget *widget,
	GdkEventButton *event,
	gpointer data)
{
	event->button = 2;
	return false;
}

gboolean iMedia::seekbar_change_cb(GtkRange * range,
	GtkScrollType scroll,
	gdouble value,
	gpointer data)
{
	((iMedia*)data)->set_position(value);
	return false;
}

gboolean iMedia::timeout_cb( gpointer data )
{
	iMedia *media = (iMedia*)data;
	media->update_position( media->get_position() );

    if(media->get_position() == media->get_duration())
    {
        media->block_slideshow(true); //Stop blocking the slideshow
    }

	if(!media->duration)
		media->update_duration(media->get_duration());
	return true;
}

void iMedia::f_unrealize_cb( GtkWidget *widget, gpointer data )
{
	((iMedia*)data)->f_visible = false;
}

void iMedia::activate_media_keys( void )
{
	DBusGConnection *connection;
	GError *err = NULL;
	DBusGProxy *proxy;

	connection = dbus_g_bus_get (DBUS_BUS_SESSION, &err);
	if (connection == NULL) {
		g_warning ("Error connecting to D-Bus: %s", err->message);
		return;
	}

	/* Try the gnome-settings-daemon version,
	 * then the gnome-control-center version of things */
	proxy = dbus_g_proxy_new_for_name_owner (connection,
		"org.gnome.SettingsDaemon",
		"/org/gnome/SettingsDaemon/MediaKeys",
		"org.gnome.SettingsDaemon.MediaKeys", NULL);
	if (proxy == NULL) {
		proxy = dbus_g_proxy_new_for_name_owner (connection,
		"org.gnome.SettingsDaemon",
		"/org/gnome/SettingsDaemon",
		"org.gnome.SettingsDaemon", &err);
	}

	dbus_g_connection_unref (connection);
	
	if (err != NULL) {
		g_warning ("Failed to create dbus proxy for org.gnome.SettingsDaemon: %s", err->message);
		g_error_free (err);
	} else {
		dbus_proxy = proxy;
		//g_signal_connect_object (proxy, "destroy", G_CALLBACK (dbus_proxy_destroy_cb), &dbus_proxy, GConnectFlags(0));
		g_signal_connect(G_OBJECT(proxy), "destroy", G_CALLBACK (dbus_proxy_destroy_cb), &dbus_proxy);
	}

	dbus_g_proxy_call (proxy, "GrabMediaPlayerKeys", NULL,
		G_TYPE_STRING, "Gloobus-Preview", G_TYPE_UINT, 0, G_TYPE_INVALID, G_TYPE_INVALID);

	dbus_g_object_register_marshaller (media_marshal_VOID__STRING_STRING,
			G_TYPE_NONE, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INVALID);
	dbus_g_proxy_add_signal (proxy, "MediaPlayerKeyPressed",
			G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INVALID);
	dbus_g_proxy_connect_signal (proxy, "MediaPlayerKeyPressed",
			G_CALLBACK (media_key_pressed_cb), this, NULL);

	focus_in_id = g_signal_connect (G_OBJECT ( ui->get_window() ),
		"focus-in-event", G_CALLBACK (focus_in_cb), this);

	dbus_proxy = proxy;
}

void iMedia::deactivate_media_keys( void )
{
	if (dbus_proxy != NULL) {
		DBusGProxy *proxy = (DBusGProxy*)dbus_proxy;
		dbus_g_proxy_call (proxy, "ReleaseMediaPlayerKeys", NULL,
			G_TYPE_STRING, "Gloobus-Preview", G_TYPE_INVALID, G_TYPE_INVALID);
		g_object_unref (proxy);
		dbus_proxy = NULL;
	}

	if (focus_in_id) {
		g_signal_handler_disconnect (G_OBJECT (ui->get_window()), focus_in_id);
	}
}

static void
dbus_proxy_destroy_cb (DBusGProxy *proxy, gpointer data)
{
	DBusGProxy **dbus_proxy = (DBusGProxy**)data;
	*dbus_proxy = 0;
}

gboolean iMedia::
focus_in_cb( GtkWidget *window, GdkEventFocus *event, gpointer data )
{
	iMedia *media = (iMedia*)data;
	if (media->dbus_proxy) {
		DBusGProxy *proxy = (DBusGProxy*)media->dbus_proxy;
		dbus_g_proxy_call (proxy, "GrabMediaPlayerKeys", NULL,
			G_TYPE_STRING, "Gloobus-Preview", G_TYPE_UINT, 0, G_TYPE_INVALID, G_TYPE_INVALID);
	}
	return false;
}

static void
media_key_pressed_cb (DBusGProxy *proxy,
					const gchar *application,
					const gchar *key,
					gpointer data)
{
	iMedia *media = (iMedia*)data;
	if (g_strcmp0 ("Gloobus-Preview", application) == 0) {
		if (g_strcmp0 ("Play", key) == 0)
			media->play_pause();
		else if (g_strcmp0 ("Previous", key) == 0)
			media->previous();
		else if (g_strcmp0 ("Next", key) == 0)
			media->next();
		else if (g_strcmp0 ("Stop", key) == 0)
			media->stop();
	}
}

static void
media_marshal_VOID__STRING_STRING (GClosure     *closure,
                                   GValue       *return_value G_GNUC_UNUSED,
                                   guint         n_param_values,
                                   const GValue *param_values,
                                   gpointer      invocation_hint G_GNUC_UNUSED,
                                   gpointer      marshal_data)
{
	typedef void (*GMarshalFunc_VOID__STRING_STRING) (gpointer data1,
		gpointer arg_1, gpointer arg_2, gpointer data2);
	
	register GMarshalFunc_VOID__STRING_STRING callback;
	register GCClosure *cc = (GCClosure*) closure;
	register gpointer data1, data2;

	g_return_if_fail (n_param_values == 3);

	if (G_CCLOSURE_SWAP_DATA (closure)) {
		data1 = closure->data;
		data2 = param_values->data[0].v_pointer;
	} else {
		data1 = param_values->data[0].v_pointer;
		data2 = closure->data;
	}
	callback = (GMarshalFunc_VOID__STRING_STRING) (marshal_data ? marshal_data : cc->callback);

	callback (data1, (param_values+1)->data[0].v_pointer,
		(param_values+2)->data[0].v_pointer, data2);
}

void iMedia::next( void )
{
	g_idle_add(&next_file_cb, NULL);
}

void iMedia::previous( void )
{
	g_idle_add(&previous_file_cb, NULL);
}

gboolean iMedia::next_file_cb( gpointer )
{
	Gloobus::instance()->next_file();
	return false;
}

gboolean iMedia::previous_file_cb( gpointer )
{
	Gloobus::instance()->prev_file();
	return false;
}
